program Test;

{$APPTYPE CONSOLE}

uses
  SysUtils, JNI, JavaBridge;

var
  Jvm, Env: Pointer;

type
  TTest = class
  private
    FCls: JClass;
    FObj: JObject;
    FInit: JMethodID;
    FHello: JMethodID;
    FValue: JFieldID;
    function GetValue: Integer;
    procedure SetValue(Value: Integer);
  public
    constructor Create;
    procedure Hello(const Text: string);
    property Value: Integer read GetValue write SetValue;
  end;

constructor TTest.Create;
begin
  inherited Create;
  FCls := JNIEnv(Env).FindClass('test/Test');
  FInit := JNIEnv(Env).GetMethodID(FCls, '<init>', '()V');
  FHello := JNIEnv(Env).GetMethodID(FCls, 'hello', '(Ljava/lang/String;)V');
  FValue := JNIEnv(Env).GetFieldID(FCls, 'value', 'I');
  FObj := JNIEnv(Env).NewObjectA(FCls, FInit, nil);
end;

function TTest.GetValue: Integer;
begin
  Result := JNIEnv(Env).GetIntField(FObj, FValue);
end;

procedure TTest.SetValue(Value: Integer);
begin
  JNIEnv(Env).SetIntField(FObj, FValue, Value);
end;

procedure TTest.Hello(const Text: string);
begin
  JNIEnv(Env).CallNonvirtualVoidMethodA(FObj, FCls, FHello, PJValue(Args(Env, [CreateJString(Env, Text)])));
end;

var
  InitArgs: JavaVMInitArgs;
  MyTest: TTest;

begin
  JavaBridge.LoadLibrary;

  FillChar(InitArgs, SizeOf(InitArgs), 0);
  InitArgs.version := JNI_VERSION_1_6;
  if JNI_CreateJavaVM(Jvm, Env, InitArgs) <> 0 then
    raise Exception.Create('Cannot initialize JavaVM');

  MyTest := TTest.Create;
  WriteLn('Value = ' + IntToStr(MyTest.Value));
  MyTest.Value := 10;
  WriteLn('Value = ' + IntToStr(MyTest.Value));
  MyTest.Hello('Hello, world!');
end.