(*
 * Copyright (c) 1996, 2018, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 *)

(*
 * We used part of Netscape's Java Runtime Interface (JRI) as the starting
 * point of our design and implementation.
 *)

(******************************************************************************
 * Java Runtime Interface
 * Copyright (c) 1996 Netscape Communications Corporation. All rights reserved.
 *****************************************************************************)

unit JNI;

interface

{$ifdef CONDITIONALEXPRESSIONS}
  {$if CompilerVersion >= 14}
    {$define D6PLUS} // Delphi 6 or higher
  {$ifend}

  {$if CompilerVersion >= 15}
    {$define D7PLUS} // Delphi 7 or higher
  {$ifend}
{$else}
  {$define MSWINDOWS}
{$endif}

{$ifdef D7PLUS}
  {$WARN UNSAFE_TYPE OFF}
{$endif}

type
{$ifndef D6PLUS}
  PPointer = ^Pointer;
{$endif}

  JInt = Integer;
  JLong = Int64;
  JByte = ShortInt;

  JBoolean = Boolean;
  JChar = WideChar;
  JShort = SmallInt;
  JFloat = Single;
  JDouble = Double;
  JSize = JInt;

  JObject = Pointer;
  JClass = JObject;
  JThrowable = JObject;
  JString = JObject;
  JArray = JObject;
  JBooleanArray = JArray;
  JByteArray = JArray;
  JCharArray = JArray;
  JShortArray = JArray;
  JIntArray = JArray;
  JLongArray = JArray;
  JFloatArray = JArray;
  JDoubleArray = JArray;
  JObjectArray = JArray;

  JWeak = JObject;

  JValue = packed record
    case Integer of
      0: (Z: JBoolean);
      1: (B: JByte);
      2: (C: JChar);
      3: (S: JShort);
      4: (I: JInt);
      5: (J: JLong);
      6: (F: JFloat);
      7: (D: JDouble);
      8: (L: JObject);
  end;

  JFieldID = Pointer;
  JMethodID = Pointer;
  
  PJInt = ^JInt;
  PJLong = ^JLong;
  PJByte = ^JByte;
  PJBoolean = ^JBoolean;
  PJChar = ^JChar;
  PJShort = ^JShort;
  PJFloat = ^JFloat;
  PJDouble = ^JDouble;
  PJValue = ^JValue;

  VaList = Pointer;

  // Return values from jobjectRefType
  JObjectRefType = (
    JNIInvalidRefType {$IFDEF D6PLUS} = 0 {$ENDIF D6PLUS},
    JNILocalRefType {$IFDEF D6PLUS} = 1 {$ENDIF D6PLUS},
    JNIGlobalRefType {$IFDEF D6PLUS} = 2 {$ENDIF D6PLUS},
    JNIWeakGlobalRefType {$IFDEF D6PLUS} = 3 {$ENDIF D6PLUS});

const
  JNI_FALSE = 0;
  JNI_TRUE = 1;

  // possible return values for JNI functions.
const
  JNI_OK = 0; // success
  JNI_ERR = -1; // unknown error
  JNI_EDETACHED = -2; // thread detached from the VM
  JNI_EVERSION = -3; // JNI version error
  JNI_ENOMEM = -4; // not enough memory
  JNI_EEXIST = -5; // VM already created
  JNI_EINVAL = -6; // invalid arguments

  // used in ReleaseScalarArrayElements
  JNI_COMMIT = 1;
  JNI_ABORT = 2;

type
  // used in RegisterNatives to describe native method name, signature, and function pointer
  JNINativeMethod = packed record
    Name: PAnsiChar;
    Signature: PAnsiChar;
    FnPtr: Pointer;
  end;

  PJNINativeMethod = ^JNINativeMethod;

  JNIEnv = interface
    procedure reserved3; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function GetVersion: JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function DefineClass(const Name: PAnsiChar; Loader: JObject; const Buf: PJByte; Len: JSize): JClass; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function FindClass(const Name: PAnsiChar): JClass; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function FromReflectedMethod(Method: JObject): JMethodID; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function FromReflectedField(Field: JObject): JFieldID; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function ToReflectedMethod(Cls: JClass; MethodID: JMethodID; IsStatic: JBoolean): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function GetSuperclass(Sub: JClass): JClass; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function IsAssignableFrom(Sub, Sup: JClass): JBoolean; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function ToReflectedField(Cls: JClass; FieldID: JFieldID; IsStatic: JBoolean): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function Throw(Obj: JThrowable): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function ThrowNew(Clazz: JClass; const Msg: PAnsiChar): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function ExceptionOccurred: JThrowable; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure ExceptionDescribe; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure ExceptionClear; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure FatalError(const Msg: PAnsiChar); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function PushLocalFrame(Capacity: JInt): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function PopLocalFrame(Result: JObject): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function NewGlobalRef(LObj: JObject): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure DeleteGlobalRef(GRef: JObject); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure DeleteLocalRef(Obj: JObject); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function IsSameObject(Obj1, Obj2: JObject): JBoolean; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function NewLocalRef(Ref: JObject): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function EnsureLocalCapacity(Capacity: JInt): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function AllocObject(Clazz: JClass): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function NewObject(Clazz: JClass; MethodID: JMethodID { ... } ): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function NewObjectV(Clazz: JClass; MethodID: JMethodID; Args: VaList): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function NewObjectA(Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function GetObjectClass(Obj: JObject): JClass; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function IsInstanceOf(Obj: JObject; Clazz: JClass): JBoolean; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function GetMethodID(Clazz: JClass; const Name, Sig: PAnsiChar): JMethodID; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallObjectMethod(Obj: JObject; MethodID: JMethodID { ... } ): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallObjectMethodV(Obj: JObject; MethodID: JMethodID; Args: VaList): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallObjectMethodA(Obj: JObject; MethodID: JMethodID; const Args: PJValue): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallBooleanMethod(Obj: JObject; MethodID: JMethodID { ... } ): JBoolean; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallBooleanMethodV(Obj: JObject; MethodID: JMethodID; Args: VaList): JBoolean; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallBooleanMethodA(Obj: JObject; MethodID: JMethodID; const Args: PJValue): JBoolean; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallByteMethod(Obj: JObject; MethodID: JMethodID { ... } ): JByte; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallByteMethodV(Obj: JObject; MethodID: JMethodID; Args: VaList): JByte; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallByteMethodA(Obj: JObject; MethodID: JMethodID; const Args: PJValue): JByte; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallCharMethod(Obj: JObject; MethodID: JMethodID { ... } ): JChar; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallCharMethodV(Obj: JObject; MethodID: JMethodID; Args: VaList): JChar; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallCharMethodA(Obj: JObject; MethodID: JMethodID; const Args: PJValue): JChar; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallShortMethod(Obj: JObject; MethodID: JMethodID { ... } ): JShort; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallShortMethodV(Obj: JObject; MethodID: JMethodID; Args: VaList): JShort; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallShortMethodA(Obj: JObject; MethodID: JMethodID; const Args: PJValue): JShort; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallIntMethod(Obj: JObject; MethodID: JMethodID { ... } ): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallIntMethodV(Obj: JObject; MethodID: JMethodID; Args: VaList): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallIntMethodA(Obj: JObject; MethodID: JMethodID; const Args: PJValue): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallLongMethod(Obj: JObject; MethodID: JMethodID { ... } ): JLong; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallLongMethodV(Obj: JObject; MethodID: JMethodID; Args: VaList): JLong; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallLongMethodA(Obj: JObject; MethodID: JMethodID; const Args: PJValue): JLong; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallFloatMethod(Obj: JObject; MethodID: JMethodID { ... } ): JFloat; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallFloatMethodV(Obj: JObject; MethodID: JMethodID; Args: VaList): JFloat; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallFloatMethodA(Obj: JObject; MethodID: JMethodID; const Args: PJValue): JFloat; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallDoubleMethod(Obj: JObject; MethodID: JMethodID { ... } ): JDouble; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallDoubleMethodV(Obj: JObject; MethodID: JMethodID; Args: VaList): JDouble; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallDoubleMethodA(Obj: JObject; MethodID: JMethodID; const Args: PJValue): JDouble; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    procedure CallVoidMethod(Obj: JObject; MethodID: JMethodID { ... } ); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure CallVoidMethodV(Obj: JObject; MethodID: JMethodID; Args: VaList); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure CallVoidMethodA(Obj: JObject; MethodID: JMethodID; const Args: PJValue); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallNonvirtualObjectMethod(Obj: JObject; Clazz: JClass; MethodID: JMethodID { ... } ): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallNonvirtualObjectMethodV(Obj: JObject; Clazz: JClass; MethodID: JMethodID; Args: VaList): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallNonvirtualObjectMethodA(Obj: JObject; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallNonvirtualBooleanMethod(Obj: JObject; Clazz: JClass; MethodID: JMethodID { ... } ): JBoolean; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallNonvirtualBooleanMethodV(Obj: JObject; Clazz: JClass; MethodID: JMethodID; Args: VaList): JBoolean; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallNonvirtualBooleanMethodA(Obj: JObject; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JBoolean; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallNonvirtualByteMethod(Obj: JObject; Clazz: JClass; MethodID: JMethodID { ... } ): JByte; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallNonvirtualByteMethodV(Obj: JObject; Clazz: JClass; MethodID: JMethodID; Args: VaList): JByte; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallNonvirtualByteMethodA(Obj: JObject; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JByte; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallNonvirtualCharMethod(Obj: JObject; Clazz: JClass; MethodID: JMethodID { ... } ): JChar; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallNonvirtualCharMethodV(Obj: JObject; Clazz: JClass; MethodID: JMethodID; Args: VaList): JChar; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallNonvirtualCharMethodA(Obj: JObject; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JChar; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallNonvirtualShortMethod(Obj: JObject; Clazz: JClass; MethodID: JMethodID { ... } ): JShort; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallNonvirtualShortMethodV(Obj: JObject; Clazz: JClass; MethodID: JMethodID; Args: VaList): JShort; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallNonvirtualShortMethodA(Obj: JObject; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JShort; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallNonvirtualIntMethod(Obj: JObject; Clazz: JClass; MethodID: JMethodID { ... } ): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallNonvirtualIntMethodV(Obj: JObject; Clazz: JClass; MethodID: JMethodID; Args: VaList): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallNonvirtualIntMethodA(Obj: JObject; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallNonvirtualLongMethod(Obj: JObject; Clazz: JClass; MethodID: JMethodID { ... } ): JLong; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallNonvirtualLongMethodV(Obj: JObject; Clazz: JClass; MethodID: JMethodID; Args: VaList): JLong; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallNonvirtualLongMethodA(Obj: JObject; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JLong; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallNonvirtualFloatMethod(Obj: JObject; Clazz: JClass; MethodID: JMethodID { ... } ): JFloat; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallNonvirtualFloatMethodV(Obj: JObject; Clazz: JClass; MethodID: JMethodID; Args: VaList): JFloat; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallNonvirtualFloatMethodA(Obj: JObject; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JFloat; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallNonvirtualDoubleMethod(Obj: JObject; Clazz: JClass; MethodID: JMethodID { ... } ): JDouble; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallNonvirtualDoubleMethodV(Obj: JObject; Clazz: JClass; MethodID: JMethodID; Args: VaList): JDouble; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallNonvirtualDoubleMethodA(Obj: JObject; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JDouble; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    procedure CallNonvirtualVoidMethod(Obj: JObject; Clazz: JClass; MethodID: JMethodID { ... } ); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure CallNonvirtualVoidMethodV(Obj: JObject; Clazz: JClass; MethodID: JMethodID; Args: VaList); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure CallNonvirtualVoidMethodA(Obj: JObject; Clazz: JClass; MethodID: JMethodID; const Args: PJValue); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function GetFieldID(Clazz: JClass; const Name, Sig: PAnsiChar): JFieldID; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function GetObjectField(Obj: JObject; FieldID: JFieldID): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetBooleanField(Obj: JObject; FieldID: JFieldID): JBoolean; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetByteField(Obj: JObject; FieldID: JFieldID): JByte; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetCharField(Obj: JObject; FieldID: JFieldID): JChar; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetShortField(Obj: JObject; FieldID: JFieldID): JShort; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetIntField(Obj: JObject; FieldID: JFieldID): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetLongField(Obj: JObject; FieldID: JFieldID): JLong; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetFloatField(Obj: JObject; FieldID: JFieldID): JFloat; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetDoubleField(Obj: JObject; FieldID: JFieldID): JDouble; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    procedure SetObjectField(Obj: JObject; FieldID: JFieldID; Val: JObject); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetBooleanField(Obj: JObject; FieldID: JFieldID; Val: JBoolean); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetByteField(Obj: JObject; FieldID: JFieldID; Val: JByte); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetCharField(Obj: JObject; FieldID: JFieldID; Val: JChar); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetShortField(Obj: JObject; FieldID: JFieldID; Val: JShort); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetIntField(Obj: JObject; FieldID: JFieldID; Val: JInt); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetLongField(Obj: JObject; FieldID: JFieldID; Val: JLong); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetFloatField(Obj: JObject; FieldID: JFieldID; Val: JFloat); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetDoubleField(Obj: JObject; FieldID: JFieldID; Val: JDouble); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function GetStaticMethodID(Clazz: JClass; const Name, Sig: PAnsiChar): JMethodID; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallStaticObjectMethod(Clazz: JClass; MethodID: JMethodID { ... } ): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallStaticObjectMethodV(Clazz: JClass; MethodID: JMethodID; Args: VaList): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallStaticObjectMethodA(Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallStaticBooleanMethod(Clazz: JClass; MethodID: JMethodID { ... } ): JBoolean; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallStaticBooleanMethodV(Clazz: JClass; MethodID: JMethodID; Args: VaList): JBoolean; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallStaticBooleanMethodA(Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JBoolean; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallStaticByteMethod(Clazz: JClass; MethodID: JMethodID { ... } ): JByte; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallStaticByteMethodV(Clazz: JClass; MethodID: JMethodID; Args: VaList): JByte; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallStaticByteMethodA(Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JByte; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallStaticCharMethod(Clazz: JClass; MethodID: JMethodID { ... } ): JChar; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallStaticCharMethodV(Clazz: JClass; MethodID: JMethodID; Args: VaList): JChar; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallStaticCharMethodA(Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JChar; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallStaticShortMethod(Clazz: JClass; MethodID: JMethodID { ... } ): JShort; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallStaticShortMethodV(Clazz: JClass; MethodID: JMethodID; Args: VaList): JShort; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallStaticShortMethodA(Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JShort; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallStaticIntMethod(Clazz: JClass; MethodID: JMethodID { ... } ): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallStaticIntMethodV(Clazz: JClass; MethodID: JMethodID; Args: VaList): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallStaticIntMethodA(Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallStaticLongMethod(Clazz: JClass; MethodID: JMethodID { ... } ): JLong; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallStaticLongMethodV(Clazz: JClass; MethodID: JMethodID; Args: VaList): JLong; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallStaticLongMethodA(Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JLong; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallStaticFloatMethod(Clazz: JClass; MethodID: JMethodID { ... } ): JFloat; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallStaticFloatMethodV(Clazz: JClass; MethodID: JMethodID; Args: VaList): JFloat; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallStaticFloatMethodA(Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JFloat; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function CallStaticDoubleMethod(Clazz: JClass; MethodID: JMethodID { ... } ): JDouble; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallStaticDoubleMethodV(Clazz: JClass; MethodID: JMethodID; Args: VaList): JDouble; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function CallStaticDoubleMethodA(Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JDouble; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    procedure CallStaticVoidMethod(Cls: JClass; MethodID: JMethodID { ... } ); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure CallStaticVoidMethodV(Cls: JClass; MethodID: JMethodID; Args: VaList); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure CallStaticVoidMethodA(Cls: JClass; MethodID: JMethodID; const Args: PJValue); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function GetStaticFieldID(Clazz: JClass; const Name, Sig: PAnsiChar): JFieldID; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function GetStaticObjectField(Clazz: JClass; FieldID: JFieldID): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetStaticBooleanField(Clazz: JClass; FieldID: JFieldID): JBoolean; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetStaticByteField(Clazz: JClass; FieldID: JFieldID): JByte; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetStaticCharField(Clazz: JClass; FieldID: JFieldID): JChar; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetStaticShortField(Clazz: JClass; FieldID: JFieldID): JShort; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetStaticIntField(Clazz: JClass; FieldID: JFieldID): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetStaticLongField(Clazz: JClass; FieldID: JFieldID): JLong; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetStaticFloatField(Clazz: JClass; FieldID: JFieldID): JFloat; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetStaticDoubleField(Clazz: JClass; FieldID: JFieldID): JDouble; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    procedure SetStaticObjectField(Clazz: JClass; FieldID: JFieldID; Value: JObject); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetStaticBooleanField(Clazz: JClass; FieldID: JFieldID; Value: JBoolean); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetStaticByteField(Clazz: JClass; FieldID: JFieldID; Value: JByte); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetStaticCharField(Clazz: JClass; FieldID: JFieldID; Value: JChar); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetStaticShortField(Clazz: JClass; FieldID: JFieldID; Value: JShort); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetStaticIntField(Clazz: JClass; FieldID: JFieldID; Value: JInt); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetStaticLongField(Clazz: JClass; FieldID: JFieldID; Value: JLong); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetStaticFloatField(Clazz: JClass; FieldID: JFieldID; Value: JFloat); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetStaticDoubleField(Clazz: JClass; FieldID: JFieldID; Value: JDouble); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function NewString(const Unicode: PJChar; Len: JSize): JString; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetStringLength(Str: JString): JSize; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function GetStringChars(Str: JString; var IsCopy: JBoolean): PJChar; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure ReleaseStringChars(Str: JString; const Chars: PJChar); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function NewStringUTF(const Utf: PAnsiChar): JString; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetStringUTFLength(Str: JString): JSize; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function GetStringUTFChars(Str: JString; var IsCopy: JBoolean): PAnsiChar; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure ReleaseStringUTFChars(Str: JString; const Chars: PAnsiChar); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function GetArrayLength(Array_: JArray): JSize; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function NewObjectArray(Len: JSize; Clazz: JClass; Init: JObject): JObjectArray; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetObjectArrayElement(Array_: JObjectArray; Index: JSize): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetObjectArrayElement(Array_: JObjectArray; Index: JSize; Val: JObject); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function NewBooleanArray(Len: JSize): JBooleanArray; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function NewByteArray(Len: JSize): JByteArray; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function NewCharArray(Len: JSize): JCharArray; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function NewShortArray(Len: JSize): JShortArray; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function NewIntArray(Len: JSize): JIntArray; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function NewLongArray(Len: JSize): JLongArray; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function NewFloatArray(Len: JSize): JFloatArray; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function NewDoubleArray(Len: JSize): JDoubleArray; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function GetBooleanArrayElements(Array_: JBooleanArray; var IsCopy: JBoolean): PJBoolean; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetByteArrayElements(Array_: JByteArray; var IsCopy: JBoolean): PJByte; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetCharArrayElements(Array_: JCharArray; var IsCopy: JBoolean): PJChar; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetShortArrayElements(Array_: JShortArray; var IsCopy: JBoolean): PJShort; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetIntArrayElements(Array_: JIntArray; var IsCopy: JBoolean): PJInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetLongArrayElements(Array_: JLongArray; var IsCopy: JBoolean): PJLong; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetFloatArrayElements(Array_: JFloatArray; var IsCopy: JBoolean): PJFloat; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetDoubleArrayElements(Array_: JDoubleArray; var IsCopy: JBoolean): PJDouble; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    procedure ReleaseBooleanArrayElements(Array_: JBooleanArray; Elems: PJBoolean; Mode: JInt); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure ReleaseByteArrayElements(Array_: JByteArray; Elems: PJByte; Mode: JInt); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure ReleaseCharArrayElements(Array_: JCharArray; Elems: PJChar; Mode: JInt); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure ReleaseShortArrayElements(Array_: JShortArray; Elems: PJShort; Mode: JInt); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure ReleaseIntArrayElements(Array_: JIntArray; Elems: PJInt; Mode: JInt); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure ReleaseLongArrayElements(Array_: JLongArray; Elems: PJLong; Mode: JInt); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure ReleaseFloatArrayElements(Array_: JFloatArray; Elems: PJFloat; Mode: JInt); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure ReleaseDoubleArrayElements(Array_: JDoubleArray; Elems: PJDouble; Mode: JInt); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    procedure GetBooleanArrayRegion(Array_: JBooleanArray; Start, Len: JSize; Buf: PJBoolean); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure GetByteArrayRegion(Array_: JByteArray; Start, Len: JSize; Buf: PJByte); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure GetCharArrayRegion(Array_: JCharArray; Start, Len: JSize; Buf: PJChar); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure GetShortArrayRegion(Array_: JShortArray; Start, Len: JSize; Buf: PJShort); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure GetIntArrayRegion(Array_: JIntArray; Start, Len: JSize; Buf: PJInt); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure GetLongArrayRegion(Array_: JLongArray; Start, Len: JSize; Buf: PJLong); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure GetFloatArrayRegion(Array_: JFloatArray; Start, Len: JSize; Buf: PJFloat); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure GetDoubleArrayRegion(Array_: JDoubleArray; Start, Len: JSize; Buf: PJDouble); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    procedure SetBooleanArrayRegion(Array_: JBooleanArray; Start, Len: JSize; const Buf: PJBoolean); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetByteArrayRegion(Array_: JByteArray; Start, Len: JSize; const Buf: PJByte); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetCharArrayRegion(Array_: JCharArray; Start, Len: JSize; const Buf: PJChar); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetShortArrayRegion(Array_: JShortArray; Start, Len: JSize; const Buf: PJShort); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetIntArrayRegion(Array_: JIntArray; Start, Len: JSize; const Buf: PJInt); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetLongArrayRegion(Array_: JLongArray; Start, Len: JSize; const Buf: PJLong); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetFloatArrayRegion(Array_: JFloatArray; Start, Len: JSize; const Buf: PJFloat); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure SetDoubleArrayRegion(Array_: JDoubleArray; Start, Len: JSize; const Buf: PJDouble); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function RegisterNatives(Clazz: JClass; const Methods: PJNINativeMethod; NMethods: JInt): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function UnregisterNatives(Clazz: JClass): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function MonitorEnter(Obj: JObject): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function MonitorExit(Obj: JObject): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function GetJavaVM(var VM: Pointer): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    procedure GetStringRegion(Str: JString; Start, Len: JSize; Buf: PJChar); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure GetStringUTFRegion(Str: JString; Start, Len: JSize; Buf: PAnsiChar); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function GetPrimitiveArrayCritical(Array_: JArray; var IsCopy: JBoolean): Pointer; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure ReleasePrimitiveArrayCritical(Array_: JArray; Carray: Pointer; Mode: JInt); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function GetStringCritical(String_: JString; var IsCopy: JBoolean): PJChar; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure ReleaseStringCritical(String_: JString; const CString: PJChar); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function NewWeakGlobalRef(Obj: JObject): JWeak; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    procedure DeleteWeakGlobalRef(Ref: JWeak); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function ExceptionCheck: JBoolean; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    function NewDirectByteBuffer(Address: Pointer; Capacity: JLong): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetDirectBufferAddress(Buf: JObject): Pointer; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetDirectBufferCapacity(Buf: JObject): JLong; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    (* New JNI 1.6 Features *)

    function GetObjectRefType(Obj: JObject): JObjectRefType; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

    (* Module Features *)

    function GetModule(Clazz: JClass): JObject; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
  end;

(*
 * optionString may be any option accepted by the JVM, or one of the
 * following:
 *
 * -D<name>=<value>          Set a system property.
 * -verbose[:class|gc|jni]   Enable verbose output, comma-separated. E.g.
 *                           "-verbose:class" or "-verbose:gc,class"
 *                           Standard names include: gc, class, and jni.
 *                           All nonstandard (VM-specific) names must begin
 *                           with "X".
 * vfprintf                  extraInfo is a pointer to the vfprintf hook.
 * exit                      extraInfo is a pointer to the exit hook.
 * abort                     extraInfo is a pointer to the abort hook.
 *)

  JavaVMOption = packed record
    OptionString: PAnsiChar;
    ExtraInfo: Pointer;
  end;

  JavaVMInitArgs = packed record
    Version: JInt;

    NOptions: JInt;
    Options: ^JavaVMOption;
    IgnoreUnrecognized: JBoolean;
  end;

  JavaVMAttachArgs = packed record
    Version: JInt;

    Name: PAnsiChar;
    Group: JObject;
  end;

  JavaVM = interface
    function DestroyJavaVM: JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function AttachCurrentThread(out Env: Pointer; Args: Pointer): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function DetachCurrentThread: JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function GetEnv(out Env: Pointer; Version: JInt): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
    function AttachCurrentThreadAsDaemon(out Env: Pointer; Args: Pointer): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
  end;

{$ifdef MSWINDOWS}
  TJNI_GetDefaultJavaVMInitArgs = function(out Args: JavaVMInitArgs): JInt; stdcall;
  TJNI_CreateJavaVM = function(out VM: Pointer { JavaVM }; out Env: Pointer { JNIEnv }; var Args: JavaVMInitArgs): JInt; stdcall;
  TJNI_GetCreatedJavaVMs = function(out PVM: Pointer { JavaVM }; Size: JSize; out OutSize: JSize): JInt; stdcall;
{$endif MSWINDOWS}

  // Defined by native libraries
  JNI_OnLoad = function(VM: Pointer { JavaVM }; Reserved: Pointer): JInt; {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}
  JNI_OnUnload = procedure(VM: Pointer { JavaVM }; Reserved: Pointer); {$ifdef MACOS} cdecl; {$ELSE} stdcall; {$endif MACOS}

const
  JNI_VERSION_1_1 = $00010001;
  JNI_VERSION_1_2 = $00010002;
  JNI_VERSION_1_4 = $00010004;
  JNI_VERSION_1_6 = $00010006;
  JNI_VERSION_1_8 = $00010008;
  JNI_VERSION_9   = $00090000;
  JNI_VERSION_10  = $000a0000;

implementation

end.